import random

import pyglet

from prime.widgets.buttons import Button
from prime.widgets.buttons import SeqButton
from prime.widgets.panels import HPanel
from prime.widgets.panels import VPanel
from prime.widgets.panels import CenterBlock
from prime.graphics.sprites import SeqImage
from prime.system.window import get_window
from prime.graphics.prims import GRectangle
from prime.graphics.base import Graphic
from prime.widgets.buttons import ImageHoverButton

PEG_IMG = "pegs.png"
FEEDBACK_IMG = "feedback.png"
ACC_BTN_IMG = "accept-button.png"

MENU_NEWGAME_IMG = "menu-newgame.png"
MENU_SURRENDER_IMG = "menu-surrender.png"

PADDING_SMALL = 10
PADDING_BIG = 20
PADDING_FEED = 5

class Peg(SeqButton):

    def __init__(self, x=0, y=0, colors=4, enabled=False, active=False):
        super(Peg, self).__init__(x=x, y=y, rows=9, cols=3, image=PEG_IMG)

        self.color = 0
        self.max_color = colors

        self.enabled = enabled
        self.active = active

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, value):
        self._color = value
        self.img_index = (self._color, 0)

    @property
    def enabled(self): return self._enabled

    @enabled.setter
    def enabled(self, value):
        self._enabled = value
        if not self._enabled:
            self.img_index = (0, 0)
            self.color = 0
        else:
            self.img_index = (1, 0)
            self.color = 1

    def on_click(self):
        if not self.enabled or not self.active:
            return

        self.color = self.color % self.max_color + 1

    def on_mouse_press(self):
        if not self.active: return
        self.img_index = (self.color, 2)

    def on_hover(self):
        if not self.active: return
        self.img_index = (self.color, 1)

    def on_nohover(self):
        if not self.active: return
        self.img_index = (self.color, 0)

class PegRow(HPanel):

    def __init__(self, x=0, y=0, pegs=4, colors=4, enabled=False, active=False):
        super(PegRow, self).__init__(x=x, y=y, padding=PADDING_SMALL)

        self.pegs = pegs
        self.enabled = enabled
        self.active = active

        for i in range(self.pegs):
            self.add(Peg(enabled=enabled, active=active, colors=colors))

    @property
    def active(self): return self._active

    @active.setter
    def active(self, value):
        self._active = value
        for peg in self.widgets:
            peg.active = self._active

    @property
    def enabled(self):
        return self._enabled

    @enabled.setter
    def enabled(self, value):
        self._enabled = value
        for peg in self.widgets:
            peg.enabled = self._enabled

    @property
    def code(self):
        code = []
        for peg in self.widgets:
            code.append(peg.color - 1)
        return code

    # TODO: Check value!
    @code.setter
    def code(self, value):
        for i in range(len(value)):
            self.widgets[i].color = value[i]

class Feedback(HPanel):

    def __init__(self, x=0, y=0, pegs=4, padding=PADDING_FEED):
        super(Feedback, self).__init__(x, y, padding)

        odd = pegs % 2 # Peg count is odd number.
        cnt = pegs / 2 + odd # Ceiling.
        self.slots = [None for i in range(cnt * 2)]

        j = 0
        for i in range(cnt):
            slot1 = SeqImage(rows=3, image=FEEDBACK_IMG)
            slot2 = SeqImage(rows=3, image=FEEDBACK_IMG)
            
            # Add slots int correct order.
            self.slots[j] = slot2
            self.slots[len(self.slots) / 2 + j] = slot1
            
            j += 1

            panel = VPanel(padding=padding)

            panel.add(slot1)
            panel.add(slot2)

            self.add(panel)

        if odd: # Remove extra slot.
            slot = self.slots.pop()

            try:
                slot.parent.graphics.remove(slot)
                slot.parent.widgets.remove(slot)
            except ValueError: pass

    def set(self, in_place=0, in_row=0):
        for slot in self.slots[:in_place]:
            slot.img_index = 2

        for slot in self.slots[in_place:in_place + in_row]:
            slot.img_index = 1
        
        for slot in self.slots[in_place + in_row:]:
            slot.img_index = 0

class AcceptButton(SeqButton):
    
    def __init__(self, x=0, y=0, enabled=False):
        super(AcceptButton, self).__init__(x=x, y=y, rows=4, image=ACC_BTN_IMG)

        self.enabled = enabled

    @property
    def enabled(self): return self._enabled

    @enabled.setter
    def enabled(self, value):
        self._enabled = value
        if self._enabled:
            self.img_index = 1
        else:
            self.img_index = 0

    def on_mouse_press(self):
        if not self.enabled: return
        self.img_index = 3

    def on_hover(self):
        if not self.enabled: return
        self.img_index = 2

    def on_nohover(self):
        if not self.enabled: return
        self.img_index = 1

class Row(HPanel):

    def __init__(self, x=0, y=0, enabled=False, active=False, board=None,
            pegs=4, colors=4):
        super(Row, self).__init__(x, y, padding=PADDING_BIG)

        self.peg_row = PegRow(pegs=pegs, colors=colors)
        self.add(self.peg_row)

        accept_button = AcceptButton()
        self.accept_button = accept_button

        # Center-align it.
        accept_block = CenterBlock(x=20, y=20, width=accept_button.width,
                height=53, graphic=accept_button)
        self.add(accept_block)

        feedback = Feedback(pegs=pegs)
        self.feedback = feedback

        feedback_block = CenterBlock(width=feedback.width, height=53,
                graphic=feedback)
        self.add(feedback_block)

        self.enabled = enabled
        self.active = active

        self.board = board

        # Accept row action.
        def on_acc_click(button):
            if self.board is None: return
            if not button.enabled: return
            self.board.on_row_accept(self.peg_row.code)

        self.accept_button.on_click = lambda : on_acc_click(self.accept_button)

    def reset(self):
        self.peg_row.enabled = False
        self.accept_button.enabled = False
        self.feedback.set(0, 0)

    @property
    def active(self): return self._active

    @active.setter
    def active(self, value):
        self._active = value
        self.peg_row.active = value

        # Only when active and enabled.
        self.accept_button.enabled = value and self.enabled

    @property
    def enabled(self): return self._enabled

    @enabled.setter
    def enabled(self, value=True):
        self._enabled = value

        self.peg_row.enabled = value
        self.peg_row.active = value

        self.accept_button.enabled = value

class Board(VPanel):
    
    def __init__(self, x=0, y=0, padding=PADDING_SMALL, rows=8, pegs=4,
            colors=4, color_repeat=False):
        super(Board, self).__init__(x=x, y=y, padding=padding)

        self.pegs = pegs
        self.colors = colors
        self.row_cnt = rows
        self.color_repeat = color_repeat

        self.code = None # Code to guess.

        for i in range(rows):
            self.add(Row(board=self, pegs=pegs, colors=colors))

        self.c_row = 0
        self.rows[self.c_row].enabled = True

        self.set_code()

        self.result_row = None # To show correct result in case of losing.

    def set_code(self):
        self.code = []

        if not self.color_repeat:
            population = range(self.colors)
            self.code = random.sample(population, self.pegs)
        else:
            for i in range(self.pegs):
                self.code.append(random.randint(0, self.colors - 1))

    def reset(self):
        self.result_row.code = [0 for i in range(self.pegs)]

        self.c_row = 0
        for row in self.rows:
            row.reset()
        self.rows[self.c_row].enabled = True
        self.set_code()

    def show_code(self):
        self.result_row.code = [i + 1 for i in self.code]

    @property
    def active_row(self): return self.rows[self.c_row]

    def on_row_accept(self, guess):
        row = self.active_row

        in_place, in_row = guess_result(self.code, guess)

        row.feedback.set(in_place, in_row)
        row.active = False

        self.c_row += 1

        if in_place == self.pegs or self.c_row >= self.row_cnt: # Game over.
            self.show_code()
            return

        self.rows[self.c_row].enabled = True

    @property
    def rows(self): return self.widgets

class MainMenu(HPanel):

    def __init__(self, x=0, y=0, board=None):
        if board is None:
            raise Exception("Main menu must have a board.")
        super(MainMenu, self).__init__(x=x, y=y, padding=PADDING_SMALL)

        self.board = board # Game board to control.

        newgame_button = ImageHoverButton(image=MENU_NEWGAME_IMG)
        surrender_button = ImageHoverButton(image=MENU_SURRENDER_IMG)
        #settings_button = ImageHoverButton(image="menu-settings.png")

        @newgame_button.event
        def on_click():
            self.board.reset()

        @surrender_button.event
        def on_click():
            self.board.active_row.active = False
            self.board.show_code()

        self.add(newgame_button)
        self.add(surrender_button)
        #self.add(settings_button)

class MainPanel(VPanel):
    """Panel which contains board, result row and control buttons."""

    def __init__(self, x=0, y=0, board=None):
        if board is None:
            raise Exception("MainPanel must have a board.")
        super(MainPanel, self).__init__(x=x, y=y)

        result_row = PegRow(pegs=board.pegs)
        board.result_row = result_row

        main_menu = MainMenu(board=board)

        self.add(main_menu) # Main menu.

        self.add(Graphic(width=10, height=10)) # TODO: Constants.
        
        self.add(board) # Game board.
        
        self.add(Graphic(width=10, height=5))
        self.add(GRectangle(width=self.width, height=2, color=(80, 80, 80)))
        self.add(Graphic(width=10, height=5))
        
        self.add(result_row) # Result row.  To show correct code.

def guess_result(code, guess):
    """Return guess feedback.  in_place, in_row."""

    in_place = 0
    in_row = 0

    code = list(code)
    guess = list(guess)

    i = 0
    while i < len(guess):
        if guess[i] == code[i]:
            in_place += 1
            del guess[i]
            del code[i]
            i -= 1
        i += 1

    i = 0
    while i < len(guess):
        if guess[i] in code:
            in_row += 1
            j = code.index(guess[i])
            del code[j]
            del guess[i]
            i -= 1
        i += 1

    return in_place, in_row
