import pyglet

from prime.widgets.base import Widget
from prime.widgets.panels import Panel
from prime.system.event import PanelEventHandler

class Window(pyglet.window.Window):
    """Pyglet window with event delegation to its widgets."""

    instance = None

    def __init__(self, *args, **kwargs):
        super(Window, self).__init__(*args, **kwargs)

        self.graphics = [] # All drawable objects.
        self.widgets = [] # All drawable objects with events.

        # Root panel -- for all graphics added to window.
        self.root_panel = Panel(0, 0, self.width, self.height)
        self.add(self.root_panel)

        # Event handler for panels.
        self.handler = PanelEventHandler()

        self.__class__.instance = self # Save instance.

    def on_draw(self):
        self.clear()

        for graphic in self.graphics:
            graphic.draw()

    def add(self, graphic):
        """Adds graphic object to the list."""

        if isinstance(graphic, Panel):
            self.widgets.append(graphic)

        elif isinstance(graphic, Widget):
            self.root_panel.add(graphic)

        self.graphics.append(graphic)

    def on_mouse_press(self, x, y, buttons, modifiers):
        for widget in reversed(self.widgets):
            if not widget.hits(x, y):
                continue
            
            self.handler.hit_something = False
            self.handler.event_mouse_press(widget, x, y, buttons, modifiers)
            if self.handler.hit_something:
                break

    def on_mouse_release(self, x, y, buttons, modifiers):
        for widget in reversed(self.widgets):
            if not widget.hits(x, y):
                continue
            
            self.handler.hit_something = False
            self.handler.event_mouse_release(widget, x, y, buttons, modifiers)
            if self.handler.hit_something:
                break
        
        self.handler.last_press = None

    def on_mouse_motion(self, x, y, dx, dy):
        self.handler.inside = False # Mouse is in some widget.

        for widget in self.widgets:
            if not widget.hits(x, y):
                continue
            
            self.handler.event_mouse_motion(widget, x, y, dx, dy)

        # If mouse is not in any widget, then we leave in_wdget.
        if not self.handler.inside and not self.handler.in_widget is None:
            self.handler.in_widget.mouse_leave()
            self.handler.in_widget.nohover()
            self.handler.in_widget = None

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        self.handler.inside = False

        for widget in self.widgets:
            if not widget.hits(x, y):
                continue
            
            self.handler.event_mouse_drag(widget, x, y, dx, dy, buttons,
                    modifiers)

        if not self.handler.inside and not self.handler.in_widget is None:
            self.handler.in_widget.mouse_leave()
            self.handler.in_widget.nohover()
            self.handler.in_widget = None

def get_window():
    if Window.instance is None:
        return Window()
    else:
        return Window.instance
