import pyglet

from prime.widgets.base import Widget
from prime.graphics.prims import GRectangle
from prime.graphics.sprites import Image
from prime.graphics.sprites import SeqImage
from prime.geometry.utils import is_inside
from prime.utils.descriptor import DelegateDesc

class Button(Widget):
    """Minimal button with click events."""

    def __init__(self, x=0, y=0, width=20, height=20):
        super(Button, self).__init__(x, y, width, height)

    def hits(self, x=0, y=0):
        if is_inside(self.x, self.y, self.width, self.height, x, y):
            return True

class TestButton(Button, GRectangle):
    """Button for tests."""
    
    def __init__(self, x=0, y=0, width=20, height=20, color=(153, 102, 204)):
        GRectangle.__init__(self, x=x, y=y, width=width, height=height,
                color=color)

    def on_mouse_press(self):
        self.color = (0, 221, 221)

    def on_hover(self):
        self.color = (51, 51, 153)

    def on_nohover(self):
        self.color = (153, 102, 204)

class SeqButton(Button, SeqImage):
    """Button with multiple images."""

    def __init__(self, x=0, y=0, rows=1, cols=1, image=None):
        """image -- pyglet resource path."""
        SeqImage.__init__(self, x=x, y=y, rows=rows, cols=cols, image=image)

class BlockButton(Button, GRectangle):
    """Colored button with click events."""
    
    def __init__(self, x=0, y=0, width=20, height=20, color=(255, 255, 255)):
        GRectangle.__init__(self, x=x, y=y, width=width, height=height,
                color=color)

class TextButton(Button):
    """Button with text."""

    text = DelegateDesc("document", "text")
    x = DelegateDesc("layout", "x")
    y = DelegateDesc("layout", "y")
    width = DelegateDesc("layout", "content_width")
    height = DelegateDesc("layout", "content_height")

    def __init__(self, x=0, y=0, label="label", color=(255, 0, 0)):
        self.document = pyglet.text.document.UnformattedDocument(label)
        self.document.set_style(0, 0, {"color" : color + (255,)})
        self.layout = pyglet.text.layout.TextLayout(self.document)

        self.x = x
        self.y = y

    @property
    def color(self):
        return self.document.get_style("color")

    @color.setter
    def color(self, value):
        self.document.set_style(0, 0, {"color" : value + (255,)})

    def draw(self):
        self.layout.draw()

class ImageButton(Button, Image):
    """Button with image."""

    def __init__(self, x=0, y=0, image=None):
        Image.__init__(self, x=x, y=y, image=image)

class ImageHoverButton(Button, SeqImage):
    """Image button with effects on click and hover."""

    def __init__(self, x=0, y=0, image=None):
        """image -- three vertical stacked images, from bottom -- normal, hover,
        press."""

        SeqImage.__init__(self, x=x, y=y, rows=3, image=image)

    def on_hover(self):
        self.img_index = 1

    def on_nohover(self):
        self.img_index = 0

    def on_mouse_press(self):
        self.img_index = 2

class TextBlockButton(Button):
    """TextButton with dimension."""

    def __init__(self, x=0, y=0, width=None, height=None, label="label",
            color=(255, 0, 0)):
        self.rect = GRectangle(x, y, width, height, color=(30, 30, 30))

        self.document = pyglet.text.document.UnformattedDocument(label)
        self.document.set_style(0, 0, {"color" : color + (255,)})
        self.layout = pyglet.text.layout.TextLayout(self.document)

        self._x = x
        self._y = y
        self._width = width
        self._height = height

        # Align center.
        self.layout.y = (self.y + self.height / 2 -
                self.layout.content_height / 2)
        self.layout.x = (self.x + self.width / 2 -
                self.layout.content_width / 2)
        pass

    @property
    def x(self):
        return self._x

    @x.setter
    def x(self, value):
        self._x = value
        self.layout.x = (self.x + self.width / 2 -
                self.layout.content_width / 2)
        self.rect.x = value

    @property
    def y(self):
        return self._y

    @y.setter
    def y(self, value):
        self._y = value
        self.layout.y = (self.y + self.height / 2 -
                self.layout.content_height / 2)
        self.rect.y = value

    @property
    def width(self):
        return self._width

    @width.setter
    def width(self, value):
        self._width = value
        self.rect.width = value
        self.layout.x = (self.x + self.width / 2 -
                self.layout.content_width / 2)

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, value):
        self._height = value
        self.rect.height = value
        self.layout.y = (self.y + self.height / 2 -
                self.layout.content_height / 2)

    def draw(self):
        self.rect.draw()
        self.layout.draw()
