from prime.widgets.base import Widget
from prime.geometry.utils import is_inside
from prime.graphics.prims import GRectangle

class Panel(Widget):
    """Panel which contains Graphics.  Free widget position."""

    def __init__(self, x=0, y=0, width=0, height=0):
        self._x = x
        self._y = y
        
        self._width = width
        self._height = height

        self.widgets = []
        self.graphics = []

        self.parent = None

    def add(self, graphic=None):

        if isinstance(graphic, Widget):
            self.widgets.append(graphic)

        graphic.parent = self
        self.graphics.append(graphic)

    def event_child_change(self, widget):
        """Called on child graphics parameter change -- width, height, etc."""
        pass

    @property
    def x(self):
        return self._x

    @x.setter
    def x(self, value):
        dx = value - self._x
        for g in self.graphics:
            g.x += dx

        self._x = value

    @property
    def y(self):
        return self._y

    @y.setter
    def y(self, value):
        dy = value - self._y
        for g in self.graphics:
            g.y += dy

        self._y = value

    @property
    def width(self):
        return self._width

    @width.setter
    def width(self, value):
        diff = value - self._width

        self._width = value
        if self.parent is not None:
            self.parent.event_child_change(self, width=diff)

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, value):
        diff = value - self._height

        self._height = value
        if self.parent is not None:
            self.parent.event_child_change(self, height=diff)

    def draw(self):
        for graphic in self.graphics:
            graphic.draw()

    def hits(self, x=0, y=0):
        if is_inside(self.x, self.y, self.width, self.height, x, y):
            return True

class VPanel(Panel):
    """Vertical Panel."""

    def __init__(self, x=0, y=0, padding=0):
        super(VPanel, self).__init__(x, y)
        self.padding = padding
        self.w_elem = None # Max width element.

    def add(self, graphic=None):
        super(VPanel, self).add(graphic)
    
        shift = 0 # Compensate height from padding.
        
        if len(self.graphics) > 1: # No compensation for only 1 element.
            shift = self.padding

        # Update dimensions.
        if graphic.width > self.width:
            self.width = graphic.width

        self.height += graphic.height + shift

        # Place graphic.
        graphic.x = self.x
        graphic.y = self.y + self.height - graphic.height

    def event_child_change(self, graphic, **kwargs):
        if graphic is None:
            return

        width = kwargs.get("width", None)
        height = kwargs.get("height", None)

        if width is None:
            pass
        elif self.w_elem is None or self.w_elem.width < graphic.width:
            self.width += width
            self.w_elem = graphic
        elif self.w_elem is graphic:
            self.w_elem = max(self.graphics, key=lambda g: g.width)
            self.width = self.w_elem.width

        if height is not None and height != 0:
            self.height += height # Update height.

            index = self.graphics.index(graphic)
            for gr in self.graphics[index + 1:]: # Update graphic position.
                gr.y += height

class HPanel(Panel):
    """Horizontal Panel."""

    def __init__(self, x=0, y=0, padding=0):
        super(HPanel, self).__init__(x, y)
        self.padding = padding
        self.h_elem = None # Max height element.

    def add(self, graphic=None):
        super(HPanel, self).add(graphic)

        shift = 0

        if len(self.graphics) > 1:
            shift = self.padding

        if graphic.height > self.height:
            self.height = graphic.height

        self.width += graphic.width + shift

        graphic.x = self.x + self.width - graphic.width
        graphic.y = self.y

    def event_child_change(self, graphic, **kwargs):
        if graphic is None:
            return

        width = kwargs.get("width", None)
        height = kwargs.get("height", None)

        if height is None:
            pass
        elif self.h_elem is None or self.h_elem.height < graphic.height:
            self.height += height
            self.h_elem = graphic
        elif self.h_elem is graphic:
            self.h_elem = max(self.graphics, key=lambda g: g.height)
            self.height = self.h_elem.height

        if width is not None and width != 0:
            self.width += width # Update height.

            index = self.graphics.index(graphic)
            for gr in self.graphics[index + 1:]: # Update graphic position.
                gr.x += width

class Block(Panel):
    """Container for one graphic."""

    def __init__(self, x=0, y=0, width=0, height=0, graphic=None):
        #self.graphic = graphic
        super(Block, self).__init__(x=x, y=y, width=width, height=height)
        self.add(graphic)

    def add(self, graphic=None):
        if len(self.graphics) > 1:
            raise Exception("Block can not contain more than 1 graphic.")

        super(Block, self).add(graphic)

        graphic.x = self.x
        graphic.y = self.y
        
class CenterBlock(Block):
    
    def add(self, graphic=None):
        super(CenterBlock, self).add(graphic)

        graphic.x = self.x + (self.width - graphic.width) / 2.0
        graphic.y = self.y + (self.height - graphic.height) / 2.0
